function IFrameChannel(id) {
    this._constructor(id);
}
(function() {

    var DEFAULT_ID = "iframechannel";
    var BLANK_URL = "about:blank";

    IFrameChannel.prototype = {

        id: null,
        name: null,
        url: null,
        preventCaching: null,
        iframe: null,

        _constructor: function(id) {
            this.id = id || DEFAULT_ID;
            this.name = this.id;
            this.url = BLANK_URL;
            this.preventCaching = false;
        },

        setName: function(name) {
            this.name = name;
        },

        setPreventCaching: function(value) {
            this.preventCaching = !!value;
        },

        submit: function(form) {
            if (form == null) {
                return;
            }

            if (this.iframe == null) {
                this.iframe = this._create();
            }

            form.target = this.iframe.name;

            // crash in Safari without delay
            if (Utils.isSafariNavigator()) {
                (function(f) { f.submit(); return false; }.delay(50, form));
            } else {
                form.submit();
            }
            return false;
        },

        open: function(url) {
            this.url = url;
            this.iframe = this._create();
        },

        send: function(data) {
            this.iframe.src = this._createUrl(data);
        },

        close: function() {
            if (this.iframe == null) {
                return;
            }

            this.iframe.src = BLANK_URL;
            var element = this.iframe;
            this.iframe = null;
//            element.parentNode.removeChild(element);
            document.body.removeChild(element.parentNode);
        },

        asElement: function() {
            return this.iframe;
        },

        _createUrl: function(data) {
            if (this.preventCaching) {
                data["nocache"] = new Date().getTime();
            }

            var queryString = this._toQueryString(data);
            var uri = encodeURI(this.url);
            if (queryString.length > 0) {
                uri += (uri.indexOf("?") == -1 ? "?" : "&") + queryString;
            }

            return uri;
        },

        _toQueryString: function(data) {
            var pairs = [];
            for (var key in data) {
                if (!data.hasOwnProperty(key)) {
                    continue;
                }
                var value = data[key];
                if (this._isArray(value)) {
                    for (var i = 0; i < value.length; ++i) {
                        pairs.push(key + "=" + encodeURIComponent(value[i]));
                    }
                } else {
                    pairs.push(key + "=" + encodeURIComponent(value));
                }
            }
            return pairs.join("&");
        },

        _isArray: function(obj) {
            return obj != null && typeof obj == "object" && 'splice' in obj && 'join' in obj;
        },

        _create: function() {
            var result = $(this.id);

            if (result == null) {
                var holder = document.body.appendChild(document.createElement(Tag.DIV));
                holder.innerHTML = "<iframe src='" + BLANK_URL + "' "
                                   + "id='" + this.id + "' name='" + this.name + "' "
                                   + "frameborder='0' " // border bug in IE7
                                   + "style='width:0;height:0;border:none;'></iframe>";
                result = holder.getElementsByTagName("iframe")[0];
                Utils.setStyle(result, {position: "relative", top: 0, left: 0});
            }

            return result;
        }
    };
})();