function UploadForm(uploader, url) {
    this._constructor(uploader, url);
}
(function() { // create local scope;

    UploadForm.URL = "";
    UploadForm.Messages = {TITLE: "Form Data", INVALID_REQUIRED: "Please provide all required fields."};

    UploadForm.prototype = {

        url: null,
        uploader: null,
        dialog: null,
        channel: null,
        forms: null,
        requiredInputNames: null,
        isFormSelection: null,
        selectedFormName: null,
        error: null,

        _constructor: function(uploader, url) {
            this.uploader = uploader;
            this.url = url || UploadForm.URL;
            this.channel = new IFrameChannel("uploadformchannel");
            this.channel.setPreventCaching(true);
            this.forms = new Array();
            this.requiredInputNames = new Array();
            this.isFormSelection = false;
        },

        init: function() {
            if (this.dialog == null) {
                this.dialog = new UploadFormDialog(this);

                for (var i = 0; i < this.forms.length; ++i) {
                    var form = this.forms[i];
                    this.dialog.addForm(form.name, form.description);
                }
            }
        },

        addForm: function(name, description) {
            this.forms.push({name: name, description: description});
        },

        getRequiredInputNames: function(formName) {
            for (var i = 0; i < this.requiredInputNames.length; ++i) {
                var entry = this.requiredInputNames[i];
                if (entry.form == formName) {
                    return entry.inputNames;
                }
            }
            return [];
        },

        setRequiredInputNames: function(formName, inputNames) {
            this.requiredInputNames.push({form: formName, inputNames: [].concat(inputNames)});
        },

        onStart: function() {
            this.error = null;

            if (this.forms.length != 0) {
                this.onStartFormSelection();
            } else {
                this.onFinish();
            }
        },

        onFinish: function() {
            if (this.dialog != null) {
                this.dialog.setVisible(false);
            }

            if (this.error != null) {
                alert(this.error);
                this.error = null;
            } else {
                this.uploader.upload();
            }
        },

        onError: function(message) {
            this.error = message;
        },

        onSubmit: function() {
            if (this.error != null) {
                this.onFinish();
                return;
            }

            if (this.isFormSelection) {
                var filesForUpload = this.uploader.getFilenames();
                this.onFinishFormSelection(this.dialog.getSelectedForm(), filesForUpload);
            } else {
                this.onFormSubmit(this._getFormNamed(this.selectedFormName));
            }
        },

        onCancel: function() {
            if (this.error != null || this.isFormSelection) {
                this.onFinish();
            } else {
                this.onFormCancel();
            }
        },

        onStartFormSelection: function() {
            this.isFormSelection = true;

            this.init();
            this.dialog.firstPage();
            this.dialog.setTitle(UploadForm.Messages.TITLE);
            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, true);
            this.dialog.setVisible(true);
        },

        onFinishFormSelection: function(formName, files) {
            this.isFormSelection = false;
            this.selectedFormName = formName;

            this.channel.open(this.url);
            this.dialog.setContent(this.channel.asElement());
            var viewport = document.viewport.getDimensions();
            this.dialog.setSize(viewport.width * 0.5, viewport.height * 0.6);
            this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, false);

            this.channel.send({form: formName, filename: files});
        },

        onCancelFormSelection: function() {
            this.isFormSelection = false;

            this.dialog.setVisible(false);
            this.onFinish();
        },

        onFormLoaded: function() {
            Utils.setStyle(this.channel.asElement(), {width: "100%", height: "100%", display: "block"});
            var form = this._getFormNamed(this.selectedFormName);

            if (form != null) {
                var thisObj = this;
                form.onsubmit = function() { return thisObj.onFormSubmit(this); };
                this.dialog.setButtonEnabled(ModalDialog.BUTTON_OK, true);
            }
        },

        onFormSubmit: function(form) {
            if (form == null) {
                return false;
            }

            if (!this._isValidForm(form)) {
                alert(UploadForm.Messages.INVALID_REQUIRED);
                return false;
            }

            form.submit();
        },

        onFormCancel: function() {
            this.onStartFormSelection();
        },

        _getFormNamed: function(name) {
            var forms = this.channel.asElement().contentWindow.document.forms;
            for (var i = 0; i < forms.length; ++i) {
                var candidate = forms[i];
                if (candidate.name == name) {
                    return candidate;
                }
            }
            return null;
        },

        _isValidForm: function(form) {
            var requiredNames = this.getRequiredInputNames(form.name);
            if (requiredNames.length == 0) {
                return true;
            }

            for (var i = 0; i < form.elements.length; ++i) {
                var input = form.elements[i];

                if (Tag.INPUT_TYPE_HIDDEN == input.type
                    || requiredNames.indexOf(input.name) == -1) {
                    continue;
                }

                if (Utils.isRadioInput(input) && form.elements[input.name].length > 1) {
                    var radioGroup = form.elements[input.name];
                    if (!this._hasSelectedItem(radioGroup)) {
                        return false;
                    }
                } else {
                    var value = $F(input) || "";
                    if (value.trim().length == 0) {
                        return false;
                    }
                }
            }
            return true;
        },

        _hasSelectedItem: function(items) {
            for (var i = 0; i < items.length; ++i) {
                if (items[i].checked) {
                    return true;
                }
            }
            return false;
        }
    };
})();